/*
  Setup Phaser engine

  Helper methods for game.js

  This loads after game.js


  Global Static class 'GG' are engine stored variables
    that are always safe to use in gg-launcher

  Global Static class 'Game' has init(), loop(),
    and any 'magic' methods defined in the bottom of this file

*/
(function(){


  var game
    , bg
    , hero
    , walls = []
    , enemies = []
    , powerups = []
    , gems = []

    // powerups
    , powerup_frozen = false
    , powerup_speed = false
    , powerup_slow = false

    // dom
    , title
    , game_ui
    , score_ui
    , game_over_screen;


  // bring global Game into scope of window.onload
  var _Game = Game;

  window.onload = function(){

    // dom
    title = document.getElementById("title");
    game_over_screen = document.getElementById("game_over_screen");
    game_ui = document.getElementById("game_ui");
    score_ui = document.getElementById("score_ui");

    if(Game.title !== undefined) title.innerHTML = Game.title;


    // engine
    enchant(); // initialize
    game = new Core(STAGE_WIDTH, STAGE_HEIGHT); // game stage
    game.fps = 60;
    game.powerups = powerups;
    game.walls = walls;
    game.enemies = enemies;
    game.gems = gems;
    GG.game = game;
    preloadAssets();

    game.onload = gameInit(_Game);
    if(_Game.loop !== undefined){
      game.onenterframe = function(e){
        if(game.running && game.frame > 10){
          _Game.loop(_Game,game.frame);
        }
      }
    }

    var start_button = document.getElementById("start");
    var body = document.getElementsByTagName("body")[0];
    start_button.onclick = function(){
      // remove title screen
      var title_screen = document.getElementById("title_screen");
      body.removeChild(title_screen);
      // start game
      game.start();
    }

  }

  function preloadAssets(){
    // gamedev overrides
    if(_Game.bg !== undefined) GG.game_bg = _Game.bg;
    if(_Game.hero_skin !== undefined) GG.hero_skin = _Game.hero_skin;
    if(_Game.hero_attack_skin !== undefined) GG.hero_attack_skin = _Game.hero_attack_skin;
    if(_Game.hero_sword_skin !== undefined) GG.hero_sword_skin = _Game.hero_sword_skin;
    if(_Game.enemy_skin !== undefined) GG.enemy_skin = _Game.enemy_skin;
    if(_Game.gem_skin !== undefined) GG.gem_skin = _Game.gem_skin;
    if(_Game.wall_skin !== undefined) GG.wall_skin = _Game.wall_skin;
    if(_Game.powerup_skin_freeze !== undefined) GG.powerup_skin_freeze = _Game.powerup_skin_freeze;
    if(_Game.powerup_skin_speed !== undefined) GG.powerup_skin_speed = _Game.powerup_skin_speed;
    if(_Game.powerup_skin_slow !== undefined) GG.powerup_skin_slow = _Game.powerup_skin_slow;

    // preload all the things!
    game.preload(GG.game_bg);
    game.preload(GG.hero_skin);
    game.preload(GG.hero_attack_skin);
    game.preload(GG.hero_sword_skin);
    game.preload(GG.gem_skin);
    game.preload(GG.enemy_skin);
    game.preload(GG.powerup_skin_freeze);
    game.preload(GG.powerup_skin_speed);
    game.preload(GG.powerup_skin_slow);
    game.preload(GG.wall_skin);

  }

  function gameInit(Game){
    
    function hero_attack () {
      if(this.attacking <= 0 && this.attack_delay <= 0){
        this.image = game.assets[GG.hero_attack_skin];
        this.attacking = GG.hero_attack_duration;
        this.attack_delay = GG.hero_attack_delay;
        this.sword.ttl = GG.hero_sword_distance;
        this.sword.x = this.x;
        this.sword.y = this.y + (SPRITE_HEIGHT/3);
        this.sword.scaleX = this.scaleX;
        this.scene.addChild(this.sword);
        // see this.enterframe handler #ATTACK_COMPLETE
      }
    }
    function hero_attack_complete () {
      this.image = game.assets[GG.hero_skin];
    }
    function hero_sword_complete () {
      this.attacking = 0;
      this.scene.removeChild(this.sword);
    }
    function enemy_die (sword_direction, sword_x) {
      if(this.death_roll < 0){
        console.log(sword_direction, sword_x, this.x);
        this.roll_direction = (sword_direction > 0 && sword_x < this.x)? 1 : -1;
        this.death_roll = ENEMY_DEATH_ROLL_DISTANCE;
      }
    }
    return function(){ // rescoped Game so it doesn't _confuse me

      bg = new Sprite(STAGE_WIDTH,STAGE_HEIGHT);
      bg.image = game.assets[GG.game_bg];
      game.rootScene.addChild(bg);

      // from game.js
      Game.init(Game);

      // load objects loaded from Game.init
      // load hero
      if(GG.hero != null){
        // settings
        if(Game.hero.speed !== undefined) GG.hero_speed = Game.hero.speed;
        Game.hero.speed = GG.hero_speed; // set default, for powerup resets

        if(Game.hero.attack_duration !== undefined) GG.hero_attack_duration = Game.hero.attack_duration;
        Game.hero.attack_duration = GG.hero_attack_duration;

        if(Game.hero.sword_distance !== undefined) GG.hero_sword_distance = Game.hero.sword_distance;
        Game.hero.sword_distance = GG.hero_sword_distance;

        if(Game.hero.attack_delay !== undefined) GG.hero_attack_delay = Game.hero.attack_delay;
        Game.hero.attack_delay = GG.hero_attack_delay;

        // init
        hero = new Sprite(SPRITE_WIDTH,SPRITE_HEIGHT);
        hero.x = GG.hero.x*TILE_SIZE;
        hero.y = GG.hero.y*TILE_SIZE;
        hero.vx = hero.vy = 0;
        hero.ax = hero.ay = 0;
        hero.walkRot = 0; // stepper for hero rotation while moving
        hero.walkHopY = 0; // stepper for hero y hopping while moving

        hero.image = game.assets[GG.hero_skin];
        hero.attack = hero_attack;
        hero.attack_complete = hero_attack_complete;
        hero.attacking = 0; // > 0 is attacking, see GG.hero_attack_duration
        hero.attack_delay = 0;
        hero.sword = new Sprite(38,16);
        hero.sword.image = game.assets[GG.hero_sword_skin];
        hero.sword.ttl = 0;
        hero.sword_complete = hero_sword_complete;

        game.rootScene.addChild(hero);

        // add keyboard controls
        game.keybind(32, 'space');
        hero.addEventListener('enterframe', function(e){
          this.vx = this.vy = 0;
          this.ax = this.ay = 0;
          var moving_h = false
            , moving_v = false;

          if(this.attacking <= 0){
            if (game.input.left){
              this.ax = -GG.hero_speed;
              moving_h = true;
            }
            if (game.input.right){
              this.ax = GG.hero_speed;
              moving_h = true;
            }
            if (game.input.up){
              this.ay = -GG.hero_speed;
              moving_v = true;
            }
            if (game.input.down){
              this.ay = GG.hero_speed;
              moving_v = true;
            }
            if (game.input.space){
              this.attack();
            }
          }


          this.vx = ((moving_h)? 0 : this.vx) + this.ax;
          this.vy = ((moving_v)? 0 : this.vy) + this.ay;

          // orthagonal movement goes faster
          if(moving_h && moving_v){
            this.vx /= DIAGONAL_MOVEMENT;
            this.vy /= DIAGONAL_MOVEMENT;
          }

          // funner walking
          if(moving_h || moving_v){
            this.rotation = Math.cos(++this.walkRot)  * 10; // 100 tumbles
            this.y += Math.sin(--this.walkHopY) * 3;
          }else{
            this.rotation = 0;
          }

          // flip on x axis
          if(this.ax < 0 && this.scaleX > 0){
            this.scaleX = -1;
          }else if(this.ax > 0 && this.scaleX < 0){
            this.scaleX = 1;
          }

          // don't move through walls
          // all this ent1 ent2 stuff is to grant the ability to move in two directions and scrub the wall
          for (var i = 0; i < walls.length; i++) {
            var ent1 = new Entity();
            ent1.x      = this.x+this.vx;
            ent1.y      = this.y;
            ent1.width  = SPRITE_WIDTH;
            ent1.height = SPRITE_HEIGHT;
            game.rootScene.addChild(ent1);
            if(walls[i].intersect(ent1)){
                this.vx -= this.vx;
            }
            game.rootScene.removeChild(ent1);
            var ent2 = new Entity();
            ent2.x      = this.x;
            ent2.y      = this.y+this.vy;
            ent2.width  = SPRITE_WIDTH;
            ent2.height = SPRITE_HEIGHT;
            game.rootScene.addChild(ent2);
            if(walls[i].intersect(ent2)){
                this.vy -= this.vy;
            }
            game.rootScene.removeChild(ent2);
          }

          // attacking #ATTACK_COMPLETE
          if (this.attacking > 0) {
            // #SLIDE this.vx += this.attacking * .05 * this.scaleX;
            if(--this.attacking == 0){
              this.attack_complete();
            }
          }
          if(this.attack_delay > 0){
            this.attack_delay--;
          }
          // sword flying
          if (this.sword.ttl > 0) {
            // account for direction
            // ease this.sword.x += this.sword.ttl * .25 * this.sword.scaleX;
            this.sword.x += 10.25 * this.sword.scaleX;
            if(--this.sword.ttl == 0){
              this.sword_complete();
            }
          }


          // movement, within stage
          if(this.x+this.vx >= 0 && this.x+this.vx <= STAGE_WIDTH-TILE_SIZE){
            this.x += this.vx;
          }else{
            this.vx = 0;
          }
          if(this.y+this.vy >= 0 && this.y+this.vy <= STAGE_HEIGHT-TILE_SIZE){
            this.y += this.vy;
          }else{
            this.vy = 0;
          }


          // intersection and collision

          for (var i = gems.length; i >= 0; i--) {
            if(this.intersect(gems[i])){
              pickupGem(i,gems[i]);
            }
          }

          for (var i = powerups.length; i >= 0; i--) {
            if(this.intersect(powerups[i])){
              pickupPowerup(powerups[i]);
            }
          }

          for (var i = 0; i < enemies.length; i++) {
            if(this.sword.ttl > 0 && this.sword.intersect(enemies[i])){
              //destroyEnemy(enemies[i]);
              enemies[i].die(this.sword.scaleX, this.sword.x);
              this.sword_complete();
            }else if(this.intersect(enemies[i])){
              doGameOver();
            }
          }

        });

      }


      // load enemies
      if(GG.enemies.length > 0){
        for (var i = 0; i < GG.enemies.length; i++) {

          // init
          var enemy = new Sprite(SPRITE_WIDTH,SPRITE_HEIGHT);
          enemy.x = GG.enemies[i].x * TILE_SIZE;
          enemy.y = GG.enemies[i].y * TILE_SIZE;
          enemy.vx = (GG.enemies[i].direction == "x")? GG.enemies[i].speed : 0;
          enemy.vy = (GG.enemies[i].direction == "y")? GG.enemies[i].speed : 0;
          enemy.ax = enemy.ay = 0;

          // ##TODO check for absolute url
          enemy.image = game.assets[GG.enemy_skin];
          enemy.death_roll = -1; // roll before death, 0 means complete
          enemy.die = enemy_die;

          game.rootScene.addChild(enemy);

          enemy.addEventListener('enterframe', function(e){
            if(this.death_roll >= 0){
              this.rotation = this.death_roll * 10; // Math.cos(this.death_roll) * 10;
              this.x += this.death_roll * .01 * this.roll_direction;
              if(this.death_roll-- == 0){
                removeEnemy(this);
              }
            }else
            if(!powerup_frozen){
              if(this.x < 0 || this.x >= STAGE_WIDTH-TILE_SIZE)  this.vx *= -1;
              if(this.y < 0 || this.y >= STAGE_HEIGHT-TILE_SIZE) this.vy *= -1;
              this.x += this.vx;
              this.y += this.vy;

              for (var i = 0; i < walls.length; i++) {
                if(this.intersect(walls[i])){
                  this.vx *= -1;
                  this.vy *= -1;
                }
              }
            }
          });

          enemies.push(enemy);
        }
      }

      // load gems
      /*
      if(GG.gems.length > 0){
        for (var i = 0; i < GG.gems.length; i++) {

          // init
          var gem = new Sprite(SPRITE_WIDTH,SPRITE_HEIGHT);
          gem.x = GG.gems[i].x * TILE_SIZE;
          gem.y = GG.gems[i].y * TILE_SIZE;

          // ##TODO check for absolute url
          gem.image = game.assets[GG.gem_skin];

          game.rootScene.addChild(gem);

          gems.push(gem);

        }
      }*/

      // load powerups
      if(GG.powerups.length > 0){
        // console.log("loading initial powerups");
        for (var i = 0; i < GG.powerups.length; i++) {
          internal_spawnPowerup(GG.powerups[i],true); // in setter-upper
        }
      }

    }
  }

  function pickupGem(i, gem){
    // feedback

    // scoring
    doScore();

    // remove gem
    for (var i = 0; i < gems.length; i++) {
      if(gems[i] == gem){
        game.rootScene.removeChild(gem);
        gems.splice(i,1);
        GG.gems.splice(i,1);
      }
    }
    // game.rootScene.removeChild(gem);
    // gems.splice(i,1);
    // GG.gems.splice(i,1);

    /* alternative
    // actually just move it somewhere else
    GG.gems[i].x = Math.floor(Math.random()*TILES_COUNT);
    GG.gems[i].y = Math.floor(Math.random()*TILES_COUNT);
    // don't spawn gem on a wall or other gem
    GG.gems[i] = rerollPosition("gem",GG.gems[i]);

    gem.x = GG.gems[i].x*TILE_SIZE;
    gem.y = GG.gems[i].y*TILE_SIZE;
    */


  }

  function removeEnemy (enemy) {
    for (var i = 0; i < enemies.length; i++) {
      if(enemies[i] == enemy){
        enemies.splice(i,1);
        GG.enemies.splice(i,1);
        game.rootScene.removeChild(enemy);
      }
    }
  }

  function pickupPowerup(powerup) {
    // resolve powerup
    switch(powerup.type){
      case Item.freeze:
        powerupFreeze();
        break;
      case Item.speed:
        powerupSpeed();
        break;
      case Item.slow:
        powerupSlow();
        break;
    }


    // remove powerup
    for (var i = 0; i < powerups.length; i++) {
      if(powerups[i] == powerup){
        game.rootScene.removeChild(powerup);
        powerups.splice(i,1);
        GG.powerups.splice(i,1);
      }
    }
  }

  function doGameOver() {
    game.stop();
    game_over_screen.style.display = "block";
    document.getElementById('game_over_score_ui').innerHTML = "Score: "+_Game.score;

    // clear all intervals
    for (var i = 0; i < GG.intervals.length; i++) {
      clearTimeout(GG.intervals[i]);
    }
  }

  function doGameWin() {
    game.stop();
    game_over_screen.style.display = "block";
    document.getElementById('game_over_score_ui').innerHTML = "Score: "+_Game.score;

    // clear all intervals
    for (var i = 0; i < GG.intervals.length; i++) {
      clearTimeout(GG.intervals[i]);
    }
  }

  function doScore() {
    // tracked in Game.score
    _Game.score++;
    score_ui.innerHTML = _Game.score;
  }

  /*
    POWERUPS
  */
  function powerupFreeze() {
    // freeze all enemies for 5 seconds
    powerup_frozen = true;
    setTimeout(function(){
      powerup_frozen = false
    },5000);
    }
  function powerupSpeed() {
    // double speed for 5 seconds
    powerup_speed = true;
    GG.hero_speed = _Game.hero.speed * 2;
    setTimeout(function(){
      powerup_speed = false
      GG.hero_speed = _Game.hero.speed;
    },5000);
  }
  function powerupSlow() {
    // half speed for 5 seconds
    powerup_slow = true;
    GG.hero_speed = _Game.hero.speed / 1.5;
    setTimeout(function(){
      powerup_slow = false
      GG.hero_speed = _Game.hero.speed;
    },5000);
  }

  /*
    Add magic methods to Game static class
  */

  Game.win = function(message) {
    var msg = message || GG.win_message;

    doGameWin();

    document.getElementById('game_over_message').innerHTML = msg;
  }
})();
