/*
  Helper methods for game.js

  This loads before game.js

  Globals for game.js


  There is a coupling issue with the Arrays
  GG.enemies and GG.game.enemies
  GG.enemies contain enemies that gamedev wants to spawn at init
  GG.game.enemies are instantiated Entities

*/

var STAGE_WIDTH = 600
  , STAGE_HEIGHT = 600
  , TILES_COUNT = 12
  , SPRITE_WIDTH = 40
  , SPRITE_HEIGHT = 40
  , ORTHAGONAL_MOVEMENT = 1
  , DIAGONAL_MOVEMENT = 1.41421356237  // divide by sqrt(2)
  , TILE_SIZE = Math.floor(STAGE_WIDTH / TILES_COUNT)
  , ENEMY_DEATH_ROLL_DISTANCE = 60; // how long enemy rolls backwards

var GG = {
  game : null,
  game_bg : 'assets/game_tile.png',
  hero : null,
  hero_skin : 'assets/graphic-07.png',
  hero_attack_skin : 'assets/graphic-29.png',
  hero_sword_skin : 'assets/sword-01.png',
  hero_speed : 5, // this actual player speed in runtime, Game.hero.speed is a gamedev setting
  hero_attack_duration : 5, // how many frames the sword is out
  hero_attack_delay : 50, // how many frames until sword is ready again
  hero_sword_distance : 15, // sword travel

  enemies : [],
  enemy_direction : "x", // defaults
  enemy_speed : 2,
  enemy_skin : 'assets/graphic-27.png',

  gems : [],
  gem_skin : 'assets/graphic-56.png',

  walls : [],
  wall_skin : 'assets/game_wall.png',

  powerups : [],

  powerup_skin_freeze : 'assets/graphic-50.png',
  powerup_skin_speed : 'assets/graphic-47.png',
  powerup_skin_slow : 'assets/graphic-48.png',


  intervals : [],

  win_message : 'Victory'

};

var Item = {
  freeze : "freeze",
  speed : "speed",
  slow : "slow"
}

function GGException(args, msg){
  var funcName = args.callee.toString();
  funcName = funcName.substr('function '.length);
  funcName = funcName.substr(0, funcName.indexOf('('));

  this.args = Array.prototype.slice.call(args);
  this.func = funcName;
  this.msg = msg;
  this.toString = function(){
    return "Game Dev Error in "+this.func+"() arguments: ["+this.args.toString()+"]\nError Message: "+this.msg;
  }

  // display error in debug_ui
  var debug_ui = document.getElementById('debug_ui');
  debug_ui.style.display = "block";
  var debug_message = document.getElementById('debug_message');
  debug_message.innerHTML = this.toString();
}

// doesn't work
// window.onerror = function(message, file, line, position, error) {
//   displayDebug("Javascript error in file:"+file+" on line:"+line+","+position+"\n"+message+"\n"+(error || ""));
// }


// assigns GG.hero = {}
function spawnHero(x,y){
  GG.hero = {x:x,y:y}
  return GG.hero;
}

function spawnEnemy(x,y,direction,speed){
  var enemy = {
    direction : direction || GG.enemy_direction,
    speed : speed || GG.enemy_speed,
    x : (x>-1)?x:false || Math.floor(Math.random()*TILES_COUNT),
    y : (y>-1)?y:false || Math.floor(Math.random()*TILES_COUNT)
  }
  if(!(x>-1)&&!(y>-1)) enemy = rerollPosition("enemy",enemy);

  GG.enemies.push(enemy);
  return enemy;
}

function placeWall(x,y){
  var wall = {
    x : (x>-1)?x:false || Math.floor(Math.random()*TILES_COUNT),
    y : (y>-1)?y:false || Math.floor(Math.random()*TILES_COUNT)
  }
  if(!(x>-1)&&!(y>-1)) wall = rerollPosition("wall",wall);

  internal_spawnWall(wall);

  return wall;
}

function placeGem(x,y) {
  var gem = {
    x : (x>-1)?x:false || Math.floor(Math.random()*TILES_COUNT),
    y : (y>-1)?y:false || Math.floor(Math.random()*TILES_COUNT)
  }
  if(!(x>-1)&&!(y>-1)) gem = rerollPosition("gem",gem);

  internal_spawnGem(gem);

  return gem;
}

// type:String all lowercase or
//   must be of type, Item
function placePowerUp(type,x,y) {
  if(Item[type] !== undefined){
    var powerup = {
      type : type,
      x : (x>-1)?x:false || Math.floor(Math.random()*TILES_COUNT),
      y : (y>-1)?y:false || Math.floor(Math.random()*TILES_COUNT)
    }
    if(!(x>-1)&&!(y>-1)) powerup = rerollPosition("powerup",powerup);

    if(GG.game == null){
      GG.powerups.push(powerup);
    }else{
      internal_spawnPowerup(powerup);
    }
    return powerup;
  }else{
    throw new GGException(arguments,"Item type does not exist: '"+type+"'");
  }
}

function every(ms,func) {
  GG.intervals.push(setInterval(func,ms));
}

var x = "x";
var y = "y";


// INTERNAL
function internal_spawnWall(_wall,init) {
  var wall = new Sprite(SPRITE_WIDTH,SPRITE_HEIGHT);
  wall.x = _wall.x * TILE_SIZE;
  wall.y = _wall.y * TILE_SIZE;

  // ##TODO check for absolute url
  wall.image = GG.game.assets[GG.wall_skin];

  //if(init === undefined)
  GG.walls.push(_wall);

  GG.game.rootScene.addChild(wall);

  GG.game.walls.push(wall);
}
function internal_spawnGem(_gem,init) {
  var gem = new Sprite(SPRITE_WIDTH,SPRITE_HEIGHT);
  gem.x = _gem.x * TILE_SIZE;
  gem.y = _gem.y * TILE_SIZE;

  // ##TODO check for absolute url
  gem.image = GG.game.assets[GG.gem_skin];

  GG.game.gems.push(gem);

  GG.game.rootScene.addChild(gem);

  GG.gems.push(gem);
}
function internal_spawnPowerup(_powerup,init) {
  var powerup = new Sprite(SPRITE_WIDTH,SPRITE_HEIGHT);
  powerup.type = _powerup.type;
  powerup.x = _powerup.x * TILE_SIZE;
  powerup.y = _powerup.y * TILE_SIZE;
  
  powerup.image = GG.game.assets[GG["powerup_skin_"+powerup.type]];

  // this will cause infinite loop
  if(init === undefined) GG.powerups.push(_powerup);

  GG.game.rootScene.addChild(powerup);

  GG.game.powerups.push(powerup);
}

function rerollPosition(type_thats_checking,obj){
  // check if obj is in a taken position
  var max_check = 0;
  while(!tileIsVacant(type_thats_checking,obj.x,obj.y) && max_check++ < 50){
    // console.log("tile isn't vacant, rerolling");
    obj.x = Math.floor(Math.random()*TILES_COUNT);
    obj.y = Math.floor(Math.random()*TILES_COUNT);
  }

  return obj;
}
function tileIsVacant(type_thats_checking,x,y) {

  // check if any walls or gems are here
  // console.log(GG.walls.length);
  if(type_thats_checking != "wall"){
    for (var i = 0; i < GG.walls.length; i++) {
      if(
        (GG.walls[i].x == x && GG.walls[i].y == y)
        || tileIsTouching(GG.game.walls[i],x,y) ){
        return false;
      }
    }
  }

  if(type_thats_checking != "gem"){
    for (var i = 0; i < GG.gems.length; i++) {
      if(
        (GG.gems[i].x == x && GG.gems[i].y == y)
        || tileIsTouching(GG.game.gems[i],x,y) ){
        return false;
      }
    }
  }

  if(type_thats_checking != "powerup"){
    for (var i = 0; i < GG.powerups.length; i++) {
      if(
        (GG.powerups[i].x == x && GG.powerups[i].y == y)
        || tileIsTouching(GG.game.powerups[i],x,y) ){
        return false;
      }
    }
  }

  if(type_thats_checking != "enemy"){
    for (var i = 0; i < GG.enemies.length; i++) {
      if(
        (GG.enemies[i].x == x && GG.enemies[i].y == y)
        || tileIsTouching(GG.game.enemies[i],x,y) ){
        return false;
      }
    }
  }
  return true;
}
function tileIsTouching(obj,x,y) {
  var ent1 = new Entity();
  ent1.x      = x*TILE_SIZE;
  ent1.y      = y*TILE_SIZE;
  ent1.width  = SPRITE_WIDTH;
  ent1.height = SPRITE_HEIGHT;
  GG.game.rootScene.addChild(ent1);
  var touching = obj.intersect(ent1);
  GG.game.rootScene.removeChild(ent1);
  if(touching){
      console.log("tileIsTouching: "+touching);
  }
  return touching;
}
